            .nolist
            .include "m32def.inc"
            .include "sam.inc"
            .list
;************************************
;
; if-(then)-else-end macro examples
; 
;************************************

; simple if-then-else example
            cpi   r16,0x20
            ifeq  space_chr
              ;...some code
            else  space_chr
              ;...some code
            end   space_chr

; no else part
            cpi   r16,0
            ifeq  null_chr
              ;...some code
            end   null_chr

; if after a skip - switch is active low due to a pullup
            sbic  porta,0
            ifs   switch_A
              ;...on code
            else  switch_a
              ;...off code
            end   switch_a
            
; IFxx would results in a branch out of range
            cpi   r16,27
            ifeq_far    esc_chr
              ;...some very long code
            end   esc_chr

; nesting
            cpi   r16,'a'
            ifeq  a_chr
              cpi   r17,'b'
              ifeq   b_chr
                ;...+a&+b
              else   b_chr
                ;...+a&-b
              end    b_chr
            else  a_chr
              cpi r17,'c'
              ifeq  c_chr
                ;...-a&+c
              else  c_chr
                ;...-a&-c
              end   c_chr
            end   a_chr

; 2 or more conditions are anded together (all conditions must be true)
.dseg
buffer:     .BYTE 400
end_buffer:
.cseg
            cpi   zh,high(end_buffer)
            ifeq_and    end_buffer_reached
            cpi   zl,low(end_buffer)
            ifeq        end_buffer_reached
              ldi   zh,high(buffer)       ;wrap buffer
              ldi   zl,low(buffer)
            end         end_buffer_reached
            ld    r0,z+                   ;read buffer

; 2 or more conditions are ored together (at least one condition must be true)
            cpi   r16,0
            ifeq_or     null_or_space
            cpi   r16,0x20
            ifeq        null_or_space
              ;...some code
            else        null_or_space
              ;...some code
            end         null_or_space

; when mixing IFxx_OR & IFxx_AND, the first true IFxx_OR takes the THEN path and
; the first false IFxx_AND takes the ELSE path (decreasing sequential precedence)
;
; want to or/and in any combination? - write your own IF blocks or use nesting!


; constructing ELIF or CASE blocks
            cpi   r16,'f'
            ifeq  case_f
              ;...f
            else  case_f
              cpi   r16,'h'
              ifeq  case_h
                ;...h
              else  case_h
                cpi   r16,'k'
                ifeq  case_k
                  ;...k
                else  case_k
                  cpi   r16,'m'
                  ifeq  case_m
                    ;...m
                  else  case_m
                    ;...no match / case else
                 end   case_m
               end   case_k
              end   case_h
            end   case_f

; three way compares for a state machine - only 3 compares for 7 states
; low execution time - min 6 (state 3) / max 11 cycles (state 5)
            cpi   r16,3
            iflo  state_3_lo
              cpi   r16,1
              iflo  state_0
                ;...0
              else  state_0
                ifeq  state_1
                  ;...1
                else  state_1
                  ;...2
                end   state_1
              end   state_0
            else  state_3_lo
              ifeq  state_3
                ;...3
              else  state_3
                cpi   r16,5
                iflo  state_4
                  ;...4
                else  state_4
                  ifeq  state_5
                    ;...5
                  else  state_5
                    ;...6 or higher
                  end   state_5
                end   state_4
              end   state_3
            end   state_3_lo

; when using larger numbers of states or cases use IJMP or ICALL

;************************************
;
; do-exit-loop examples
;
;************************************

; typical endless loop
            do    main
              ;...some code
            loop  main

; waiting for an event
            do    wait_xmit
              sbis  ucsra,udre      ;until usart transmit buffer empty
            loop  wait_xmit

; countdown loop
; similar to: for r16 = 40 to 1 step -1
            ldi   r16,40
            do    count_40
              dec   r16
            loopne count_40

; reading a buffer until end
; similar to: for z = buffer to (end_buffer - 1)
            ldi   zh,high(buffer)   ;init buffer pointer
            ldi   zl,low(buffer)
            do    read_buf
              ld    r0,z+
              cpi   zh,high(end_buffer)
            loopne read_buf
              cpi   zl,low(end_buffer)
            loopne read_buf

; using exit and nesting
            clr   r17         ;space counter
            do    skip_space
              do    wait_rx
                sbis  ucsra,rxc
              loop  wait_rx
              in    r16,udr
              cpi   r16,0x20
            exitne    skip_space
              inc   r17       ;count spaces
            loop  skip_space

; long loops - EXITxx branch would be out of range
            do    long_loop
              cpi   r16,27
              ifne  esc_received
            exit  long_loop
              end   esc_received
              ;...some very long code
            loop  long_loop

; long loops - LOOPxx branch would be out of range
            do    even_longer_loop
              ;...some very long code
              cpi   r16,27
              ifne  no_escape
            loop  even_longer_loop
              end   no_escape

; anded decision to exit
            do    anded_exit
              cpi   r16,1
              ifeq_and twentyone
              cpi   r17,2
              ifeq  twentyone
            exit  anded_exit
              end   twentyone
              ;...some code
            loop  anded_exit

; anded decision to loop
            do    anded_loop
              ;...some code
              cpi   r16,2
              ifeq_and fourtytwo
              cpi   r17,4
              ifeq  fourtytwo
            loop  anded_loop
              end   fourtytwo
